@@ -11,9 +11,9 @@ from .stations import stations
 
 
 def is_valid_station_id(station_id: str):
-    for line in stations.splitlines():
-        if len(line) > 0 and line[0].isdigit():
-            if line[12:18].strip() == station_id:
+    for line in stations.splitlines()[4:]:
+        if len(line) > 1:
+            if line[:6].strip() == station_id:
                 return True
     return False
 
@@ -21,16 +21,16 @@ def is_valid_station_id(station_id: str):
 def get_nearest_station_id(lat: float, lon: float):
     result = ""
     distance = 99999999
-    for line in stations.splitlines():
-        if len(line) > 0 and line[0].isdigit():
-            _lat = line[45:51].strip().split(".")
+    for line in stations.splitlines()[4:]:
+        if len(line) > 1:
+            _lat = line[33:39].strip().split(".")
             _lat = round(float(_lat[0]) + float(_lat[1]) / 60, 2)
-            _lon = line[52:59].strip().split(".")
+            _lon = line[41:47].strip().split(".")
             _lon = round(float(_lon[0]) + float(_lon[1]) / 60, 2)
             distance_temp = get_distance(lat, lon, _lat, _lon)
             if distance > distance_temp:
                 distance = distance_temp
-                result = line[12:18].strip()
+                result = line[:6].strip()
     return result
 
 
@@ -40,6 +40,17 @@ def get_distance(lat, lon, _lat, _lon):
     return math.sqrt(math.pow(lat_diff, 2) + math.pow(lon_diff, 2))
 
 
+def get_region(station_id: str):
+    for line in stations.splitlines()[4:]:
+        if len(line) > 1:
+            if line[:6].strip() == station_id:
+                region = line[53:]
+                if region != "":
+                    return region
+                return None
+    return None
+
+
 class WeatherDataType(Enum):
     CONDITION = "condition"
     TEMPERATURE = "TTT"  # Unit: K
@@ -66,6 +77,7 @@ class Weather:
     station_name = ""
     issue_time = None
     forecast_data = None
+    weather_report = None
 
     namespaces = {
         "kml": "http://www.opengis.net/kml/2.2",
@@ -104,9 +116,28 @@ class Weather:
         "95": ("lightning-rainy", 1),
     }
 
+    region_codes = {
+        "Nordrhein-Westfalen": "dweh",
+        "Niedersachsen": "dwhg",
+        "Bremen": "dwhg",
+        "Schleswig-Holstein": "dwhh",
+        "Hamburg": "dwhh",
+        "Sachsen": "dwlg",
+        "Sachsen-Anhalt": "dwlh",
+        "Thüringen": "dwli",
+        "Bayern": "dwmg",
+        "Baden-Württemberg": "dwsg",
+        "Hessen": "dwoh",
+        "Rheinland-Pfalz": "dwoi",
+        "Brandenburg": "dwpg",
+        "Berlin": "dwpg",
+        "Mecklenburg-Vorpommern": "dwph",
+    }
+
     def __init__(self, station_id):
         if is_valid_station_id(station_id):
             self.station_id = station_id
+            self.region = get_region(station_id)
         else:
             raise ValueError("Not a valid station_id")
 
@@ -357,6 +388,10 @@ class Weather:
         ):
             kml = download_latest_kml(self.station_id)
             self.parse_kml(kml)
+        if self.region is not None:
+            self.weather_report = download_weather_report(
+                self.region_codes[self.region]
+            )
 
     def get_weather_type(self, kmlTree, weatherDataType: WeatherDataType):
         """ Parses the kml-File to the requested value and returns the items as array"""
@@ -439,9 +474,16 @@ class Weather:
         rh_c3 = 241.2
         merged_list = {}
         for i in range(len(timesteps)):
-            T = temperatures[i] - 273.1
-            TD = dewpoints[i] - 273.1
-            RH = 100 * math.exp((rh_c2 * TD / (rh_c3 + TD)) - (rh_c2 * T / (rh_c3 + T)))
+            if temperatures[i] is not None and dewpoints[i] is not None:
+                T = temperatures[i] - 273.1
+                TD = dewpoints[i] - 273.1
+                RH = round(
+                    100
+                    * math.exp((rh_c2 * TD / (rh_c3 + TD)) - (rh_c2 * T / (rh_c3 + T))),
+                    1,
+                )
+            else:
+                RH = None
             item = {
                 WeatherDataType.TEMPERATURE.value: temperatures[i],
                 WeatherDataType.DEWPOINT.value: dewpoints[i],
@@ -458,21 +500,29 @@ class Weather:
                 WeatherDataType.SUN_DURATION.value: sun_dur[i],
                 WeatherDataType.SUN_IRRADIANCE.value: sun_irr[i],
                 WeatherDataType.FOG_PROBABILITY.value: fog_prop[i],
-                WeatherDataType.HUMIDITY.value: round(RH, 1),
+                WeatherDataType.HUMIDITY.value: RH,
             }
             merged_list[timesteps[i]] = item
         # print(f"temperatures: {self.forecast_data}")
         self.forecast_data = merged_list
 
+    def get_weather_report(self, shouldUpdate=False):
+        if shouldUpdate:
+            self.update()
+        return self.weather_report
+
+
+def download_weather_report(region_code):
+    url = f"https://www.dwd.de/DWD/wetter/wv_allg/deutschland/text/vhdl13_{region_code}.html"
+    headers = {
+        "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/89.0.4389.116 Safari/537.36"
+    }
+    request = requests.get(url, headers=headers)
+    return request.text
+
 
 def download_latest_kml(stationid):
-    url = (
-        "https://opendata.dwd.de/weather/local_forecasts/mos/MOSMIX_L/single_stations/"
-        + stationid
-        + "/kml/MOSMIX_L_LATEST_"
-        + stationid
-        + ".kmz"
-    )
+    url = f"https://opendata.dwd.de/weather/local_forecasts/mos/MOSMIX_L/single_stations/{stationid}/kml/MOSMIX_L_LATEST_{stationid}.kmz"
     request = requests.get(url)
     file = BytesIO(request.content)
     kmz = ZipFile(file, "r")
